###############################################################################
##  COMMON RULES                                                             ##
###############################################################################

$(TARGET_PATH):
	mkdir $@

$(TARGET_BASE): | $(TARGET_PATH)
	mkdir $@

$(TARGET_BIN): | $(TARGET_BASE)
	mkdir $@

$(TARGET_DOC): | $(TARGET_BASE)
	mkdir $@

$(TARGET_LIB): | $(TARGET_BASE)
	mkdir $@

$(TARGET_OBJ): | $(TARGET_BASE)
	mkdir $@

$(TARGET_SRC): | $(TARGET_BASE)
	mkdir $@

$(TARGET_INCL): | $(TARGET_BASE)
	mkdir $@

.PHONY: info
info:
	@echo
	@echo "  NAME:     " $(NAME) 
	@echo "  OS:       " $(OS)
	@echo "  ARCH:     " $(ARCH)
	@echo "  DISTRO:   " $(DISTRO_NAME)"-"$(DISTRO_VERS)
	@echo
	@echo "  PATHS:"
	@echo "      source:     " $(SOURCE)
	@echo "      target:     " $(TARGET_BASE)
	@echo "      includes:   " $(INC_PATH)
	@echo "      libraries:  " $(LIB_PATH)
	@echo "      submodules: " $(SUBMODULES)
	@echo
	@echo "  COMPILER:"
	@echo "      command:    " $(CC)
	@echo "      flags:      " $(CC_FLAGS) $(CFLAGS)
	@echo
	@echo "  LINKER:"
	@echo "      command:    " $(LD)
	@echo "      flags:      " $(LD_FLAGS) $(LDFLAGS)
	@echo
	@echo "  ARCHIVER:"
	@echo "      command:    " $(AR)
	@echo "      flags:      " $(AR_FLAGS) $(ARFLAGS)
	@echo

.PHONY: clean
clean: modules-clean
	@rm -rf $(TARGET)


.PHONY: $(TARGET_OBJ)/%.o
$(TARGET_OBJ)/%.o : %.c | $(TARGET_OBJ) 
	$(object)

###############################################################################
##  IMPLICIT RULES                                                           ##
###############################################################################

.PHONY: all

.DEFAULT_GOAL := all

%.o:
	$(object)

%.a: 
	$(archive)

%.so: 
	$(library)

%: 
	$(executable)

###############################################################################
##  MODULE RULES                                                             ##
###############################################################################

.PHONY: modules
modules: modules-build modules-prepare

.PHONY: modules-build
modules-build: $(MODULES:%=%-build)

.PHONY: modules-prepare
modules-prepare: $(MODULES:%=%-prepare)

.PHONY: modules-clean
modules-clean: $(MODULES:%=%-clean)

###############################################################################
##  GENERATED VERSION FILE RULES                                             ##
###############################################################################

$(TARGET_SRC)/version.c: | $(TARGET_SRC)
	@echo "/**" >>  $@
	@echo " * THIS FILE WAS AUTOMATICALLY GENERATED BY THE BUILD SYSTEM" >>  $@
	@echo " * DO NOT MODIFY THIS FILE. YOUR CHANGES WILL BE ERASED." >>  $@
	@echo " * THIS FILE MUST NEVER BE CHECKED IN TO VERSION CONTROL." >>  $@
	@echo " */" >>  $@
	@echo "char * citrusleaf_build_string = \"$(shell git describe)\";" >> $@

$(TARGET_OBJ)/version.o: $(TARGET_SRC)/version.c
	$(object)
